use std::collections::HashMap;
use std::fs::File;
use std::io::{BufRead, BufReader};
use std::path::Path;

pub type Symbols = HashMap<String, usize>;

pub fn load_symbols(filename: &Path) -> anyhow::Result<Symbols> {
    let file = File::open(filename)?;
    let reader = BufReader::new(file);

    let symbols = reader
        .lines()
        .filter_map(|l| {
            l.ok()
                .and_then(|line| {
                    let mut words = line.split_ascii_whitespace();
                    match (words.next(), words.next(), words.next()) {
                        // Get only two-word lines.
                        (Some(address), Some(symbol), None) => {
                            Some((symbol.to_owned(), address.to_owned()))
                        }

                        _ => None,
                    }
                })
                .and_then(|(symbol, mut address)| {
                    if let Some(colon_at) = address.find(':') {
                        address.remove(colon_at);
                    }

                    // Filter out the ones that don't have a hexadecimal number as the first word.
                    let snes_address = u32::from_str_radix(&address, 16).ok()?;

                    let pc_address = snes_to_pc_address(snes_address);

                    Some((symbol, pc_address))
                })
        })
        .collect();

    Ok(symbols)
}

pub fn snes_to_pc_address(snes: u32) -> usize {
    ((snes & 0x7FFF) + ((snes / 2) & 0xFF8000)) as usize
}

pub fn pc_to_snes_address(pc: usize) -> u32 {
    ((pc & 0x7FFF) + ((pc & 0xFF8000) * 2)) as u32
}
